#ifndef MODULES_LIDAR_INNO_FRONT_INCLUDE_CALLBACKPROCESSOR_H_
#define MODULES_LIDAR_INNO_FRONT_INCLUDE_CALLBACKPROCESSOR_H_

#include <vector>

#include "sdk_common/inno_lidar_packet_utils.h"
#include "FileOption.h"
#include "inno_lidar.h"

class CallbackProcessor
{
public:
  explicit CallbackProcessor(const FileOption &file_option, InnoLidar& obj);
  ~CallbackProcessor();

  void set_done()
  {
    _done = true;
  }

  bool is_done()
  {
    return _done;
  }

  void process_message(const enum InnoMessageLevel level, const enum InnoMessageCode code, const char *error_message);
  int process_data(const InnoDataPacket &pkt);
  int process_status(const InnoStatusPacket &pkt);

private:
  CallbackProcessor() = delete;
  CallbackProcessor(const CallbackProcessor &) = delete;
  CallbackProcessor &operator=(const CallbackProcessor &) = delete;

  InnoDataPacket *convert_to_xyz_pointcloud_malloced(const InnoDataPacket &src);
  void process_data_xyz_point_(const InnoDataPacket &pkt);

private:
  int64_t _current_frame = -1;
  int64_t _current_frame_sub = -1;
  int64_t _frame_so_far = -1;
  volatile bool _done = false;
  bool _is_first_data = true;
  uint64_t _sum_packet = 0;
  uint64_t _sum_point = 0;
  std::vector<char> _data_buffer;

  static const size_t _kBufSize = 1024 * 1024 * 10;
  std::vector<InnoDataPacket*> _oneFramePkts;
  uint16_t _packet_1s_window_count = 0;
  std::chrono::high_resolution_clock::time_point _startTime;
  InnoLidar& _InnoLidarObject;
};

#endif /* MODULES_LIDAR_INNO_FRONT_INCLUDE_CALLBACKPROCESSOR_H_ */
